@@ -20,6 +20,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define mscommon_h
 
 #include <ortp/ortp.h>
+#include <time.h>
 
 #define ms_malloc	ortp_malloc
 #define ms_malloc0	ortp_malloc0
@@ -43,6 +44,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define ms_cond_broadcast	ortp_cond_broadcast
 #define ms_cond_destroy		ortp_cond_destroy
 
+#if defined(_WIN32_WCE)
+time_t ms_time (time_t *t);
+#else
+#define ms_time time
+#endif
+
 #ifdef WIN32
 static inline void ms_debug(const char *fmt,...)
 {
@@ -81,7 +88,7 @@ static inline void ms_debug(const char *fmt,...)
 #define ms_thread_t		ortp_thread_t
 #define ms_thread_create 	ortp_thread_create
 #define ms_thread_join		ortp_thread_join
-#define ms_thread_exit		ortp_thread_exit
+
 
 struct _MSList {
 	struct _MSList *next;
@@ -95,10 +102,13 @@ typedef struct _MSList MSList;
 #define ms_list_next(elem) ((elem)->next)
 
 
+typedef int (*MSCompareFunc)(const void *a, const void *b);
+
 #ifdef __cplusplus
 extern "C"{
 #endif
 
+void ms_thread_exit(void* ret_val);
 MSList * ms_list_append(MSList *elem, void * data);
 MSList * ms_list_prepend(MSList *elem, void * data);
 MSList * ms_list_free(MSList *elem);
@@ -109,11 +119,11 @@ void ms_list_for_each(const MSList *list, void (*func)(void *));
 void ms_list_for_each2(const MSList *list, void (*func)(void *, void *), void *user_data);
 MSList *ms_list_remove_link(MSList *list, MSList *elem);
 MSList *ms_list_find(MSList *list, void *data);
-MSList *ms_list_find_custom(MSList *list, int (*compare_func)(const void *, const void*), void *user_data);
+MSList *ms_list_find_custom(MSList *list, MSCompareFunc compare_func, const void *user_data);
 void * ms_list_nth_data(const MSList *list, int index);
 int ms_list_position(const MSList *list, MSList *elem);
 int ms_list_index(const MSList *list, void *data);
-MSList *ms_list_insert_sorted(MSList *list, void *data, int (*compare_func)(const void *, const void*));
+MSList *ms_list_insert_sorted(MSList *list, void *data, MSCompareFunc compare_func);
 MSList *ms_list_insert(MSList *list, MSList *before, void *data);
 MSList *ms_list_copy(const MSList *list);
 
@@ -169,6 +179,8 @@ struct _MSSndCardDesc;
 
 void ms_sleep(int seconds);
 
+void ms_usleep(uint64_t usec);
+
 /**
  * The max payload size allowed.
  * Filters that generate data that can be sent through RTP should make packets
@@ -201,4 +213,8 @@ void ms_set_mtu(int mtu);
 }
 #endif
 
+
+#ifdef ANDROID
+#include "mediastreamer2/msjava.h"
+#endif
 #endif
