@@ -31,7 +31,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilerec.h"
 #include "mediastreamer2/msvolume.h"
 #include "mediastreamer2/msequalizer.h"
-#include "mediastreamer2/msspeexec.h"
 
 #ifdef INET6
 	#include <sys/types.h>
@@ -48,7 +47,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 /* this code is not part of the library itself, it is part of the mediastream program */
 void audio_stream_free(AudioStream *stream)
 {
-	if (stream->session!=NULL) rtp_session_destroy(stream->session);
+	if (stream->session!=NULL) {
+		rtp_session_unregister_event_queue(stream->session,stream->evq);
+		rtp_session_destroy(stream->session);
+	}
+	if (stream->evq) ortp_ev_queue_destroy(stream->evq);
 	if (stream->rtpsend!=NULL) ms_filter_destroy(stream->rtpsend);
 	if (stream->rtprecv!=NULL) ms_filter_destroy(stream->rtprecv);
 	if (stream->soundread!=NULL) ms_filter_destroy(stream->soundread);
@@ -63,6 +66,7 @@ void audio_stream_free(AudioStream *stream)
 	if (stream->ticker!=NULL) ms_ticker_destroy(stream->ticker);
 	if (stream->read_resampler!=NULL) ms_filter_destroy(stream->read_resampler);
 	if (stream->write_resampler!=NULL) ms_filter_destroy(stream->write_resampler);
+	if (stream->dtmfgen_rtp!=NULL) ms_filter_destroy(stream->dtmfgen_rtp);
 	ms_free(stream);
 }
 
@@ -107,7 +111,8 @@ static void audio_stream_configure_resampler(MSFilter *resampler,MSFilter *from,
 	ms_filter_call_method(to,MS_FILTER_GET_SAMPLE_RATE,&to_rate);
 	ms_filter_call_method(resampler,MS_FILTER_SET_SAMPLE_RATE,&from_rate);
 	ms_filter_call_method(resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&to_rate);
-	ms_debug("configuring from rate[%i] to rate [%i]",from_rate,to_rate);
+	ms_message("configuring %s-->%s from rate[%i] to rate [%i]",
+	           from->desc->name, to->desc->name, from_rate,to_rate);
 }
 
 RtpSession * create_duplex_rtpsession( int locport, bool_t ipv6){
@@ -143,6 +148,15 @@ bool_t audio_stream_alive(AudioStream * stream, int timeout){
 	RtpSession *session=stream->session;
 	const rtp_stats_t *stats=rtp_session_get_stats(session);
 	if (stats->recv!=0){
+		if (stream->evq){
+			OrtpEvent *ev=ortp_ev_queue_get(stream->evq);
+			if (ev!=NULL){
+				if (ortp_event_get_type(ev)==ORTP_EVENT_RTCP_PACKET_RECEIVED){
+					stream->last_packet_time=ms_time(NULL);
+				}
+				ortp_event_destroy(ev);
+			}
+		}
 		if (stats->recv!=stream->last_packet_count){
 			stream->last_packet_count=stats->recv;
 			stream->last_packet_time=ms_time(NULL);
@@ -194,7 +208,6 @@ static void payload_type_changed(RtpSession *session, unsigned long data){
 	audio_stream_change_decoder(stream,pt);
 }
 
-
 int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char *remip,int remport,
 	int rem_rtcp_port, int payload,int jitt_comp, const char *infile, const char *outfile,
 	MSSndCard *playcard, MSSndCard *captcard, bool_t use_ec)
@@ -203,6 +216,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	PayloadType *pt;
 	int tmp;
 	MSConnectionHelper h;
+	int sample_rate;
 
 	rtp_session_set_profile(rtps,profile);
 	if (remport>0) rtp_session_set_remote_addr_full(rtps,remip,remport,rem_rtcp_port);
@@ -218,7 +232,6 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	stream->dtmfgen=ms_filter_new(MS_DTMF_GEN_ID);
 	rtp_session_signal_connect(rtps,"telephone-event",(RtpCallback)on_dtmf_received,(unsigned long)stream);
 	rtp_session_signal_connect(rtps,"payload_type_changed",(RtpCallback)payload_type_changed,(unsigned long)stream);
-
 	/* creates the local part */
 	if (captcard!=NULL) stream->soundread=ms_snd_card_create_reader(captcard);
 	else {
@@ -238,6 +251,18 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_error("audiostream.c: undefined payload type.");
 		return -1;
 	}
+	if (rtp_profile_get_payload_from_mime (profile,"telephone-event")==NULL
+	    && ( strcasecmp(pt->mime_type,"pcmu")==0 || strcasecmp(pt->mime_type,"pcma")==0)){
+		/*if no telephone-event payload is usable and pcma or pcmu is used, we will generate
+		  inband dtmf*/
+		stream->dtmfgen_rtp=ms_filter_new (MS_DTMF_GEN_ID);
+	}
+	
+	if (ms_filter_call_method(stream->rtpsend,MS_FILTER_GET_SAMPLE_RATE,&sample_rate)!=0){
+		ms_error("Sample rate is unknown for RTP side !");
+		return -1;
+	}
+	
 	stream->encoder=ms_filter_create_encoder(pt->mime_type);
 	stream->decoder=ms_filter_create_decoder(pt->mime_type);
 	if ((stream->encoder==NULL) || (stream->decoder==NULL)){
@@ -246,30 +271,10 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		return -1;
 	}
 
-	if (use_ec) {
-		stream->ec=ms_filter_new(MS_SPEEX_EC_ID);
-		ms_filter_call_method(stream->ec,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
-		if (stream->ec_tail_len!=0)
-			ms_filter_call_method(stream->ec,MS_SPEEX_EC_SET_TAIL_LENGTH,&stream->ec_tail_len);
-		if (stream->ec_delay!=0)
-			ms_filter_call_method(stream->ec,MS_SPEEX_EC_SET_DELAY,&stream->ec_delay);
-		if (stream->ec_framesize!=0)
-			ms_filter_call_method(stream->ec,MS_SPEEX_EC_SET_FRAME_SIZE,&stream->ec_framesize);
-	}
-
-	if (stream->el_type!=ELInactive || stream->use_gc || stream->use_ng){
-		stream->volsend=ms_filter_new(MS_VOLUME_ID);
-		stream->volrecv=ms_filter_new(MS_VOLUME_ID);
-		if (stream->el_type!=ELInactive){
-			if (stream->el_type==ELControlSpeaker)
-				ms_filter_call_method(stream->volrecv,MS_VOLUME_SET_PEER,stream->volsend);
-			else ms_filter_call_method(stream->volsend,MS_VOLUME_SET_PEER,stream->volrecv);
-		}
-		if (stream->use_ng){
-			int tmp=1;
-			ms_filter_call_method(stream->volsend,MS_VOLUME_ENABLE_NOISE_GATE,&tmp);
-		}
-	}
+	stream->volsend=ms_filter_new(MS_VOLUME_ID);
+	stream->volrecv=ms_filter_new(MS_VOLUME_ID);
+	audio_stream_enable_echo_limiter(stream,stream->el_type);
+	audio_stream_enable_noise_gate(stream,stream->use_ng);
 
 	if (stream->use_agc){
 		int tmp=1;
@@ -279,20 +284,39 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	}
 
 	/* give the sound filters some properties */
-	if (ms_filter_call_method(stream->soundread,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate) != 0) {
+	if (ms_filter_call_method(stream->soundread,MS_FILTER_SET_SAMPLE_RATE,&sample_rate) != 0) {
 		/* need to add resampler*/
 		if (stream->read_resampler == NULL) stream->read_resampler=ms_filter_new(MS_RESAMPLE_ID);
 	}
 
-	if (ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate) != 0) {
+	if (ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_SAMPLE_RATE,&sample_rate) != 0) {
 		/* need to add resampler*/
 		if (stream->write_resampler == NULL) stream->write_resampler=ms_filter_new(MS_RESAMPLE_ID);
 	}
 
-
 	tmp=1;
 	ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_NCHANNELS, &tmp);
 
+	/*configure the echo canceller if required */
+	if (use_ec) {
+		stream->ec=ms_filter_new(MS_SPEEX_EC_ID);
+		ms_filter_call_method(stream->ec,MS_FILTER_SET_SAMPLE_RATE,&sample_rate);
+		if (stream->ec_tail_len!=0)
+			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_TAIL_LENGTH,&stream->ec_tail_len);
+		if (stream->ec_delay!=0){
+			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_DELAY,&stream->ec_delay);
+		}else{
+			/*configure from latency of sound card in case it is availlable */
+			int latency=0;
+			ms_filter_call_method(stream->soundread,MS_FILTER_GET_LATENCY,&latency);
+			latency-=30; /*keep 30 milliseconds security margin*/
+			if (latency<0) latency=0;
+			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_DELAY,&latency);
+		}
+		if (stream->ec_framesize!=0)
+			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_FRAMESIZE,&stream->ec_framesize);
+	}
+
 	/* give the encoder/decoder some parameters*/
 	ms_filter_call_method(stream->encoder,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
 	ms_message("Payload's bitrate is %i",pt->normal_bitrate);
@@ -329,6 +353,8 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_connection_helper_link(&h,stream->ec,1,1);
 	if (stream->volsend)
 		ms_connection_helper_link(&h,stream->volsend,0,0);
+	if (stream->dtmfgen_rtp)
+		ms_connection_helper_link(&h,stream->dtmfgen_rtp,0,0);
 	ms_connection_helper_link(&h,stream->encoder,0,0);
 	ms_connection_helper_link(&h,stream->rtpsend,0,-1);
 
@@ -431,10 +457,14 @@ AudioStream *audio_stream_new(int locport, bool_t ipv6){
 	AudioStream *stream=(AudioStream *)ms_new0(AudioStream,1);
 	stream->session=create_duplex_rtpsession(locport,ipv6);
 	stream->rtpsend=ms_filter_new(MS_RTP_SEND_ID);
+	stream->evq=ortp_ev_queue_new();
+	rtp_session_register_event_queue(stream->session,stream->evq);
 	stream->play_dtmfs=TRUE;
 	stream->use_gc=FALSE;
 	stream->use_agc=FALSE;
 	stream->use_ng=FALSE;
+	ms_filter_enable_statistics(TRUE);
+	ms_filter_reset_statistics();
 	return stream;
 }
 
@@ -459,6 +489,13 @@ void audio_stream_set_echo_canceller_params(AudioStream *st, int tail_len_ms, in
 
 void audio_stream_enable_echo_limiter(AudioStream *stream, EchoLimiterType type){
 	stream->el_type=type;
+	if (stream->volsend){
+		bool_t enable_noise_gate = stream->el_type==ELControlFull;
+		ms_filter_call_method(stream->volrecv,MS_VOLUME_ENABLE_NOISE_GATE,&enable_noise_gate);
+		ms_filter_call_method(stream->volsend,MS_VOLUME_SET_PEER,type!=ELInactive?stream->volrecv:NULL);
+	} else {
+		ms_warning("cannot set echo limiter to mode [%i] because no volume send",type);
+	}
 }
 
 void audio_stream_enable_gain_control(AudioStream *stream, bool_t val){
@@ -471,6 +508,13 @@ void audio_stream_enable_automatic_gain_control(AudioStream *stream, bool_t val)
 
 void audio_stream_enable_noise_gate(AudioStream *stream, bool_t val){
 	stream->use_ng=val;
+	if (stream->volsend){
+		ms_filter_call_method(stream->volsend,MS_VOLUME_ENABLE_NOISE_GATE,&val);
+	} else {
+		ms_warning("cannot set noise gate mode to [%i] because no volume send",val);
+	}
+
+
 }
 
 void audio_stream_set_mic_gain(AudioStream *stream, float gain){
@@ -516,6 +560,8 @@ void audio_stream_stop(AudioStream * stream)
 			ms_connection_helper_unlink(&h,stream->ec,1,1);
 		if (stream->volsend!=NULL)
 			ms_connection_helper_unlink(&h,stream->volsend,0,0);
+		if (stream->dtmfgen_rtp)
+			ms_connection_helper_unlink(&h,stream->dtmfgen_rtp,0,0);
 		ms_connection_helper_unlink(&h,stream->encoder,0,0);
 		ms_connection_helper_unlink(&h,stream->rtpsend,0,-1);
 
@@ -536,6 +582,7 @@ void audio_stream_stop(AudioStream * stream)
 
 	}
 	audio_stream_free(stream);
+	ms_filter_log_statistics();
 }
 
 RingStream * ring_start(const char *file, int interval, MSSndCard *sndcard){
@@ -546,34 +593,72 @@ RingStream * ring_start_with_cb(const char *file,int interval,MSSndCard *sndcard
 {
 	RingStream *stream;
 	int tmp;
+	int srcrate,dstrate;
+	MSConnectionHelper h;
+
 	stream=(RingStream *)ms_new0(RingStream,1);
 	stream->source=ms_filter_new(MS_FILE_PLAYER_ID);
-	if (ms_filter_call_method(stream->source,MS_FILE_PLAYER_OPEN,(void*)file)<0){
-		ms_filter_destroy(stream->source);
-		ms_free(stream);
-		return NULL;
-	}
+	if (file)
+		ms_filter_call_method(stream->source,MS_FILE_PLAYER_OPEN,(void*)file);
+	
 	ms_filter_call_method(stream->source,MS_FILE_PLAYER_LOOP,&interval);
 	ms_filter_call_method_noarg(stream->source,MS_FILE_PLAYER_START);
 	if (func!=NULL)
 		ms_filter_set_notify_callback(stream->source,func,user_data);
+	stream->gendtmf=ms_filter_new(MS_DTMF_GEN_ID);
+	
+	
 	stream->sndwrite=ms_snd_card_create_writer(sndcard);
-	ms_filter_call_method(stream->source,MS_FILTER_GET_SAMPLE_RATE,&tmp);
-	ms_filter_call_method(stream->sndwrite,MS_FILTER_SET_SAMPLE_RATE,&tmp);
+	ms_filter_call_method(stream->source,MS_FILTER_GET_SAMPLE_RATE,&srcrate);
+	ms_filter_call_method(stream->gendtmf,MS_FILTER_SET_SAMPLE_RATE,&srcrate);
+	ms_filter_call_method(stream->sndwrite,MS_FILTER_SET_SAMPLE_RATE,&srcrate);
+	ms_filter_call_method(stream->sndwrite,MS_FILTER_GET_SAMPLE_RATE,&dstrate);
+	if (srcrate!=dstrate){
+		stream->write_resampler=ms_filter_new(MS_RESAMPLE_ID);
+		ms_filter_call_method(stream->write_resampler,MS_FILTER_SET_SAMPLE_RATE,&srcrate);
+		ms_filter_call_method(stream->write_resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&dstrate);
+		ms_message("configuring resampler from rate[%i] to rate [%i]", srcrate,dstrate);
+	}
 	ms_filter_call_method(stream->source,MS_FILTER_GET_NCHANNELS,&tmp);
+	ms_filter_call_method(stream->gendtmf,MS_FILTER_SET_NCHANNELS,&tmp);
 	ms_filter_call_method(stream->sndwrite,MS_FILTER_SET_NCHANNELS,&tmp);
 	stream->ticker=ms_ticker_new();
 	ms_ticker_set_name(stream->ticker,"Audio (ring) MSTicker");
-	ms_filter_link(stream->source,0,stream->sndwrite,0);
+
+	ms_connection_helper_start(&h);
+	ms_connection_helper_link(&h,stream->source,-1,0);
+	ms_connection_helper_link(&h,stream->gendtmf,0,0);
+	if (stream->write_resampler)
+		ms_connection_helper_link(&h,stream->write_resampler,0,0);
+	ms_connection_helper_link(&h,stream->sndwrite,0,-1);
 	ms_ticker_attach(stream->ticker,stream->source);
 	return stream;
 }
 
+void ring_play_dtmf(RingStream *stream, char dtmf, int duration_ms){
+	if (duration_ms>0)
+		ms_filter_call_method(stream->gendtmf, MS_DTMF_GEN_PLAY, &dtmf);
+	else ms_filter_call_method(stream->gendtmf, MS_DTMF_GEN_START, &dtmf);
+}
+
+void ring_stop_dtmf(RingStream *stream){
+	ms_filter_call_method_noarg(stream->gendtmf, MS_DTMF_GEN_STOP);
+}
+
 void ring_stop(RingStream *stream){
+	MSConnectionHelper h;
 	ms_ticker_detach(stream->ticker,stream->source);
-	ms_filter_unlink(stream->source,0,stream->sndwrite,0);
+
+	ms_connection_helper_start(&h);
+	ms_connection_helper_unlink(&h,stream->source,-1,0);
+	ms_connection_helper_unlink(&h,stream->gendtmf,0,0);
+	if (stream->write_resampler)
+		ms_connection_helper_unlink(&h,stream->write_resampler,0,0);
+	ms_connection_helper_unlink(&h,stream->sndwrite,0,-1);
+
 	ms_ticker_destroy(stream->ticker);
 	ms_filter_destroy(stream->source);
+	ms_filter_destroy(stream->gendtmf);
 	ms_filter_destroy(stream->sndwrite);
 	ms_free(stream);
 #ifdef _WIN32_WCE
@@ -585,10 +670,10 @@ void ring_stop(RingStream *stream){
 
 int audio_stream_send_dtmf(AudioStream *stream, char dtmf)
 {
-	if (stream->rtpsend)
+	if (stream->dtmfgen_rtp)
+		ms_filter_call_method(stream->dtmfgen_rtp,MS_DTMF_GEN_PLAY,&dtmf);
+	else if (stream->rtpsend)
 		ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_SEND_DTMF,&dtmf);
-	if (stream->dtmfgen)
-		ms_filter_call_method(stream->dtmfgen,MS_DTMF_GEN_PUT,&dtmf);
 	return 0;
 }
 
