@@ -21,6 +21,23 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/mscommon.h"
 
 static MSList *desc_list=NULL;
+static bool_t statistics_enabled=FALSE;
+static MSList *stats_list=NULL;
+
+static int compare_stats_with_name(const MSFilterStats *stat, const char *name){
+	return strcmp(stat->name,name);
+}
+
+static MSFilterStats *find_or_create_stats(MSFilterDesc *desc){
+	MSList *elem=ms_list_find_custom(stats_list,(MSCompareFunc)compare_stats_with_name,desc->name);
+	MSFilterStats *ret=NULL;
+	if (elem==NULL){
+		ret=ms_new0(MSFilterStats,1);
+		ret->name=desc->name;
+		stats_list=ms_list_append(stats_list,ret);
+	}else ret=(MSFilterStats*)elem->data;
+	return ret;
+}
 
 void ms_filter_register(MSFilterDesc *desc){
 	if (desc->id==MS_FILTER_NOT_SET_ID){
@@ -31,7 +48,15 @@ void ms_filter_register(MSFilterDesc *desc){
 }
 
 void ms_filter_unregister_all(){
-	if (desc_list!=NULL) ms_list_free(desc_list);
+	if (desc_list!=NULL) {
+		ms_list_free(desc_list);
+		desc_list=NULL;
+	}
+	if (stats_list!=NULL){
+		ms_list_for_each(stats_list,ms_free);
+		ms_list_free(stats_list);
+		stats_list=NULL;
+	}
 }
 
 bool_t ms_filter_codec_supported(const char *mime){
@@ -83,6 +108,10 @@ MSFilter *ms_filter_new_from_desc(MSFilterDesc *desc){
 	obj->desc=desc;
 	if (desc->ninputs>0)	obj->inputs=(MSQueue**)ms_new0(MSQueue*,desc->ninputs);
 	if (desc->noutputs>0)	obj->outputs=(MSQueue**)ms_new0(MSQueue*,desc->noutputs);
+
+	if (statistics_enabled){
+		obj->stats=find_or_create_stats(desc);
+	}
 	if (obj->desc->init!=NULL)
 		obj->desc->init(obj);
 	return obj;
@@ -136,9 +165,9 @@ int ms_filter_link(MSFilter *f1, int pin1, MSFilter *f2, int pin2){
 
 int ms_filter_unlink(MSFilter *f1, int pin1, MSFilter *f2, int pin2){
 	MSQueue *q;
-	ms_message("ms_filter_unlink: %s:%p,%i-->%s:%p,%i",f1->desc->name,f1,pin1,f2->desc->name,f2,pin2);
 	ms_return_val_if_fail(f1, -1);
 	ms_return_val_if_fail(f2, -1);
+	ms_message("ms_filter_unlink: %s:%p,%i-->%s:%p,%i",f1->desc->name,f1,pin1,f2->desc->name,f2,pin2);
 	ms_return_val_if_fail(pin1<f1->desc->noutputs, -1);
 	ms_return_val_if_fail(pin2<f2->desc->ninputs, -1);
 	ms_return_val_if_fail(f1->outputs[pin1]!=NULL,-1);
@@ -151,6 +180,7 @@ int ms_filter_unlink(MSFilter *f1, int pin1, MSFilter *f2, int pin2){
 }
 
 #define MS_FILTER_METHOD_GET_FID(id)	(((id)>>16) & 0xFFFF)
+#define MS_FILTER_METHOD_GET_INDEX(id) ( ((id)>>8) & 0XFF) 
 
 static inline bool_t is_interface_method(unsigned int magic){
 	return magic==MS_FILTER_BASE_ID || magic>MSFilterInterfaceBegin;
@@ -174,7 +204,8 @@ int ms_filter_call_method(MSFilter *f, unsigned int id, void *arg){
 			return methods[i].method(f,arg);
 		}
 	}
-	if (magic!=MS_FILTER_BASE_ID) ms_error("no such method on filter %s",f->desc->name);
+	if (magic!=MS_FILTER_BASE_ID) ms_error("no such method on filter %s, fid=%i method index=%i",f->desc->name,magic,
+	                           MS_FILTER_METHOD_GET_INDEX(id) );
 	return -1;
 }
 
@@ -201,9 +232,50 @@ void ms_filter_destroy(MSFilter *f){
 }
 
 
+
+static uint64_t get_cur_time_ns(void)
+{
+#if defined(_WIN32_WCE)
+	DWORD timemillis = GetTickCount();
+	return (uint64_t)timemillis*1000000;
+#elif defined(WIN32)
+	return timeGetTime()*1000000LL ;
+#elif defined(__MACH__) && defined(__GNUC__) && (__GNUC__ >= 3)
+	struct timeval tv;
+	gettimeofday(&tv, NULL);
+	return (tv.tv_sec*1000000000LL) + (tv.tv_usec*1000LL);
+#elif defined(__MACH__)
+	struct timespec ts;
+	struct timeb time_val;
+
+	ftime (&time_val);
+	ts.tv_sec = time_val.time;
+	ts.tv_nsec = time_val.millitm * 1000000;
+	return (ts.tv_sec*1000000000LL) + ts.tv_nsec;
+#else
+	struct timespec ts;
+	if (clock_gettime(CLOCK_MONOTONIC,&ts)<0){
+		ms_fatal("clock_gettime() doesn't work: %s",strerror(errno));
+	}
+	return (ts.tv_sec*1000000000LL) + ts.tv_nsec;
+#endif
+}
+
+
 void ms_filter_process(MSFilter *f){
+	uint64_t start=0,stop;
 	ms_debug("Executing process of filter %s:%p",f->desc->name,f);
+
+	if (f->stats)
+		start = get_cur_time_ns();
+
 	f->desc->process(f);
+	if (f->stats){
+		stop = get_cur_time_ns();
+		f->stats->count++;
+		f->stats->elapsed+=stop-start;
+	}
+
 }
 
 void ms_filter_preprocess(MSFilter *f, struct _MSTicker *t){
@@ -305,4 +377,48 @@ int ms_connection_helper_unlink(MSConnectionHelper *h, MSFilter *f, int inpin, i
 	return err;
 }
 
+void ms_filter_enable_statistics(bool_t enabled){
+	statistics_enabled=enabled;
+}
+
+const MSList * ms_filter_get_statistics(void){
+	return stats_list;
+}
+
+void ms_filter_reset_statistics(void){
+	MSList *elem;
+	
+	for(elem=stats_list;elem!=NULL;elem=elem->next){
+		MSFilterStats *stats=(MSFilterStats *)elem->data;
+		stats->elapsed=0;
+		stats->count=0;
+	}
+}
+
+static int usage_compare(const MSFilterStats *s1, const MSFilterStats *s2){
+	if (s1->elapsed==s2->elapsed) return 0;
+	if (s1->elapsed<s2->elapsed) return 1;
+	return -1;
+}
+
+
+void ms_filter_log_statistics(void){
+	MSList *sorted=NULL;
+	MSList *elem;
+	uint64_t total=1;
+	ms_message("Filter usage statistics:");
+	for(elem=stats_list;elem!=NULL;elem=elem->next){
+		MSFilterStats *stats=(MSFilterStats *)elem->data;
+		sorted=ms_list_insert_sorted(sorted,stats,(MSCompareFunc)usage_compare);
+		total+=stats->elapsed;
+	}
+	ms_message("Name\tCount\tTime/tick (ms)\tCPU Usage");
+	for(elem=sorted;elem!=NULL;elem=elem->next){
+		MSFilterStats *stats=(MSFilterStats *)elem->data;
+		double percentage=100.0*((double)stats->elapsed)/(double)total;
+		double tpt=((double)stats->elapsed*1e-6)/((double)stats->count+1.0);
+		ms_message("%s %i %g %g",stats->name,stats->count,tpt,percentage);
+	}
+	ms_list_free(sorted);
+}
 
